const CommonSelectors = require('../lib/common_selectors');
const doubleClickZoom = require('../lib/double_click_zoom');
const Constants = require('../constants');
const isEventAtCoordinates = require('../lib/is_event_at_coordinates');

const DrawRectangle = {};

DrawRectangle.onSetup = function() {
  const rectangle = this.newFeature({
    type: Constants.geojsonTypes.FEATURE,
    properties: {
      id:(new Date()).getTime(),
      type:Constants.geojsonTypes.RECTANGLE,
      draw_type:"rectangle"
    },
    geometry: {
      type: Constants.geojsonTypes.POLYGON,
      coordinates: [[]]
    }
  }, Constants.geojsonTypes.RECTANGLE);
  const currentVertexPosition = 0;

  this.addFeature(rectangle);

  this.clearSelectedFeatures();
  doubleClickZoom.disable(this);
  this.updateUIClasses({ mouse: Constants.cursors.ADD });
  this.activateUIButton(Constants.types.POLYGON);
  this.setActionableState({
    combineFeatures: false,
    uncombineFeatures: false,
    trash: true
  });

  return {
    rectangle,
    currentVertexPosition
  };
};


DrawRectangle.clickAnywhere = function(state, e) {
  if (state.currentVertexPosition > 0 && isEventAtCoordinates(e, state.rectangle.coordinates[0][state.currentVertexPosition - 1])) {
    return this.changeMode(Constants.modes.SIMPLE_SELECT, { featureIds: [state.rectangle.id] });
  }
  this.updateUIClasses({ mouse: Constants.cursors.ADD });
  if (state.currentVertexPosition < 2) {
    state.rectangle.updateCoordinate(`0.${state.currentVertexPosition}`, e.lngLat.lng, e.lngLat.lat);
    state.currentVertexPosition++;
  } else {
    state.currentVertexPosition = 4;
  }
};

DrawRectangle.onMouseMove = function(state, e) {
  if (state.currentVertexPosition === 2) {
    const coordinates = state.rectangle.coordinates[0];
    const rectVertex = state.rectangle.getRectVertex(this.map.project(coordinates[0]), this.map.project(coordinates[1]), this.map.project(e.lngLat));
    if (rectVertex) {
      const p3 = this.map.unproject(rectVertex[2]);
      const p4 = this.map.unproject(rectVertex[3]);
      state.rectangle.updateCoordinate("0.2", p3.lng, p3.lat);
      state.rectangle.updateCoordinate("0.3", p4.lng, p4.lat);
    }
  } else if (state.currentVertexPosition === 4) {
    this.map.fire(Constants.events.CREATE, {
      features: [state.rectangle.toGeoJSON()]
    });
    this.changeMode(Constants.modes.SIMPLE_SELECT, { featureIds: [state.rectangle.id] });
  } else {
    state.rectangle.updateCoordinate(`0.${state.currentVertexPosition}`, e.lngLat.lng, e.lngLat.lat);
  }
  if (CommonSelectors.isVertex(e)) {
    this.updateUIClasses({ mouse: Constants.cursors.POINTER });
  }
};

DrawRectangle.clickOnVertex = function(state) {
  return this.changeMode(Constants.modes.SIMPLE_SELECT, { featureIds: [state.rectangle.id] });
};


DrawRectangle.onTap = DrawRectangle.onClick = function(state, e) {
  if (CommonSelectors.isVertex(e)) return this.clickOnVertex(state, e);
  return this.clickAnywhere(state, e);
};

DrawRectangle.onKeyUp = function(state, e) {
  if (CommonSelectors.isEscapeKey(e)) {
    this.deleteFeature([state.rectangle.id], { silent: true });
    this.changeMode(Constants.modes.SIMPLE_SELECT);
  } else if (CommonSelectors.isEnterKey(e)) {
    this.changeMode(Constants.modes.SIMPLE_SELECT, { featureIds: [state.rectangle.id] });
  }
};

DrawRectangle.onStop = function(state) {
  this.updateUIClasses({ mouse: Constants.cursors.NONE });
  doubleClickZoom.enable(this);
  this.activateUIButton();

  // check to see if we've deleted this feature
  if (this.getFeature(state.rectangle.id) === undefined) return;

  //remove last added coordinate
  state.rectangle.removeCoordinate(`0.${state.currentVertexPosition}`);
  if (state.rectangle.isValid()) {
    this.map.fire(Constants.events.CREATE, {
      features: [state.rectangle.toGeoJSON()]
    });
  } else {
    this.deleteFeature([state.rectangle.id], { silent: true });
    this.changeMode(Constants.modes.SIMPLE_SELECT, {}, { silent: true });
  }
};
DrawRectangle.toDisplayFeatures = function(state, geojson, display) {
  const isActivePolygon = geojson.properties.id === state.rectangle.id;
  geojson.properties.active = (isActivePolygon) ? Constants.activeStates.ACTIVE : Constants.activeStates.INACTIVE;
  if (!isActivePolygon) return display(geojson);

  // Don't render a polygon until it has two positions
  // (and a 3rd which is just the first repeated)
  if (geojson.geometry.coordinates.length === 0) return;

  const coordinateCount = geojson.geometry.coordinates[0].length;

  // If we have fewer than two positions (plus the closer),
  // it's not yet a shape to render
  if (coordinateCount < 3) return;

  geojson.properties.meta = Constants.meta.FEATURE;

  /*if (coordinateCount > 4) {
    // Add a start position marker to the map, clicking on this will finish the feature
    // This should only be shown when we're in a valid spot
    callback(createVertex(rectangle.id, geojson.geometry.coordinates[0][0], '0.0', false));
    var endPos = geojson.geometry.coordinates[0].length - 3;
    callback(createVertex(rectangle.id, geojson.geometry.coordinates[0][endPos], "0."+endPos, false));
  }*/

  // If we have more than two positions (plus the closer),
  // render the Polygon
  if (coordinateCount > 3) {
    return display(geojson);
  }

  // If we've only drawn two positions (plus the closer),
  // make a LineString instead of a Polygon
  const lineCoordinates = [
    [geojson.geometry.coordinates[0][0][0], geojson.geometry.coordinates[0][0][1]], [geojson.geometry.coordinates[0][1][0], geojson.geometry.coordinates[0][1][1]]
  ];
  return display({
    type: Constants.geojsonTypes.FEATURE,
    properties: geojson.properties,
    geometry: {
      coordinates: lineCoordinates,
      type: Constants.geojsonTypes.LINE_STRING
    }
  });
};
DrawRectangle.onTrash  = function(state) {
  this.deleteFeature([state.rectangle.id], { silent: true });
  this.changeMode(Constants.modes.SIMPLE_SELECT);
};

module.exports = DrawRectangle;
